from __future__ import absolute_import, unicode_literals

from mayan.apps.appearance.classes import Icon

icon_document_version_all_signature_verify = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_signature_list = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_list = Icon(
    driver_name='fontawesome', symbol='file-signature'
)

icon_document_version_signature_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_document_version_signature_details = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_detached_create = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_download = Icon(
    driver_name='fontawesome', symbol='download'
)
icon_document_version_signature_embedded_create = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_list = Icon(
    driver_name='fontawesome', symbol='file-signature'
)
icon_document_version_signature_upload = Icon(
    driver_name='fontawesome', symbol='upload'
)
